#include "main_hal.h"

FDCAN_TxHeaderTypeDef FDCAN_TxHeader; // CAN header, used for transmission
uint8_t FDCAN_TxData[64]; // CAN data to send
FDCAN_RxHeaderTypeDef FDCAN_RxHeader; // CAN header, used for reception
uint8_t FDCAN_RxData[64]; // CAN data, after the reception

/**
 * @brief  start additional Init
 * @param  none
 * @retval none
 */
void Additional_Init ( void )
{
  Additional_ADC_Init();
  Additional_TIM6_Init(); // SW Tim
  Additional_TIM5_Init(); // OUT5
  Additional_TIM8_Init(); // LED
  Additional_CAN_Init();
}

/**
 * @brief  start additional ADC Init
 * @param  none
 * @retval none
 */
void Additional_ADC_Init ( void )
{
  HAL_ADCEx_Calibration_Start(&hadc1, ADC_SINGLE_ENDED);
  HAL_Delay(50);
}

/**
 * @brief  start additional Init for OUT5 (LS-PWM)
 * @param  none
 * @retval none
 */
void Additional_TIM5_Init ( void )
{
  HAL_TIM_Base_Start(&htim5);
  HAL_TIM_PWM_Start(&htim5, TIM_CHANNEL_3);
}

/**
 * @brief  start additional Init Softwaretimer
 * @param  none
 * @retval none
 */
void Additional_TIM6_Init ( void )
{
  HAL_TIM_Base_Start_IT(&htim6);
}

/*
 * Led timer
 */
void Additional_TIM8_Init ( void )
{
  HAL_TIM_Base_Start(&htim8);
  HAL_TIMEx_PWMN_Start(&htim8, TIM_CHANNEL_4);
}

/**
 * @brief  start additional Init for CAN_CANFD
 * @param  none
 * @retval none
 */
void Additional_CAN_Init ( void )
{
  // Change CAN to Silent mode
  HAL_GPIO_WritePin(CAN1_ON_GPIO_Port, CAN1_ON_Pin, GPIO_PIN_SET);

  /* Configure and enable Tx Delay Compensation, required for BRS mode.
     TdcOffset default recommended value: DataTimeSeg1 * DataPrescaler
     TdcFilter default recommended value: 0 */
  if ( HAL_FDCAN_ConfigTxDelayCompensation(&hfdcan1, hfdcan1.Init.DataPrescaler * hfdcan1.Init.DataTimeSeg1, 0) != HAL_OK )
  {
    Error_Handler();
  }

  if ( HAL_FDCAN_EnableTxDelayCompensation(&hfdcan1) != HAL_OK )
  {
    Error_Handler();
  }

  /* Start the FDCAN module */
  if ( HAL_FDCAN_Start(&hfdcan1) != HAL_OK )
  {
    Error_Handler();
  }

  if ( HAL_FDCAN_ActivateNotification(&hfdcan1, FDCAN_IT_RX_FIFO0_NEW_MESSAGE, 0) != HAL_OK )
  {
    Error_Handler();
  }

  // Change CAN from Silent to Active mode
  HAL_GPIO_WritePin(CAN1_ON_GPIO_Port, CAN1_ON_Pin, GPIO_PIN_RESET);
}

/**
 * @brief  Rx FIFO 0 callback.
 * @param  hfdcan: pointer to an FDCAN_HandleTypeDef structure that contains
 *         the configuration information for the specified FDCAN.
 * @param  RxFifo0ITs: indicates which Rx FIFO 0 interrupts are signalled.
 *         This parameter can be any combination of @arg FDCAN_Rx_Fifo0_Interrupts.
 * @retval None
 */
void HAL_FDCAN_RxFifo0Callback ( FDCAN_HandleTypeDef *hfdcan, uint32_t RxFifo0ITs )
{
  if ( (RxFifo0ITs & FDCAN_IT_RX_FIFO0_NEW_MESSAGE) != RESET )
  {
    if ( hfdcan->Instance == FDCAN1 )
    {
      /* Retrieve Rx messages from RX FIFO0 */
      if ( HAL_FDCAN_GetRxMessage(hfdcan, FDCAN_RX_FIFO0, &FDCAN_RxHeader, FDCAN_RxData) == HAL_OK )
      {
        switch ( FDCAN_RxHeader.Identifier )
        {
          case 0x100:
            // Add code here when CAN-ID 0x100 was recieved
            break;
        }
      }
      else
      {
        Error_Handler();
      }
    }
  }
}

/**
 * @brief  Send a CAN-message on CAN-bus
 *         Is the CANID smaller than 0x800 the standard ID frame format is used, otherwise the extended ID frame format is used.
 * @param  CANID: specifies the CANID with which the message is to be sent
 *         DLC: specifies the number of bytes that will be sent
 *         Byte0: Data in Byte 0
 *         Byte1: Data in Byte 1
 *         Byte2: Data in Byte 2
 *         Byte3: Data in Byte 3
 *         Byte4: Data in Byte 4
 *         Byte5: Data in Byte 5
 *         Byte6: Data in Byte 6
 *         Byte7: Data in Byte 7
 * @retval none
 */
void SendCAN_ClassicMessage ( uint32_t CANID, uint32_t DLC, uint8_t Byte0, uint8_t Byte1, uint8_t Byte2, uint8_t Byte3, uint8_t Byte4, uint8_t Byte5,
    uint8_t Byte6, uint8_t Byte7 )
{
  /* Prepare Tx Header */
  FDCAN_TxHeader.FDFormat = FDCAN_CLASSIC_CAN;
  FDCAN_TxHeader.TxFrameType = FDCAN_DATA_FRAME;
  FDCAN_TxHeader.ErrorStateIndicator = FDCAN_ESI_ACTIVE;
  FDCAN_TxHeader.BitRateSwitch = FDCAN_BRS_OFF;
  FDCAN_TxHeader.TxEventFifoControl = FDCAN_NO_TX_EVENTS;
  FDCAN_TxHeader.MessageMarker = 0;
  FDCAN_TxHeader.Identifier = CANID;
  FDCAN_TxHeader.IdType = FDCAN_STANDARD_ID;
  if ( CANID >= 0x800 )
    FDCAN_TxHeader.IdType = FDCAN_EXTENDED_ID;
  FDCAN_TxHeader.DataLength = DLC;

  /* Fill Data */
  FDCAN_TxData[0] = Byte0;
  FDCAN_TxData[1] = Byte1;
  FDCAN_TxData[2] = Byte2;
  FDCAN_TxData[3] = Byte3;
  FDCAN_TxData[4] = Byte4;
  FDCAN_TxData[5] = Byte5;
  FDCAN_TxData[6] = Byte6;
  FDCAN_TxData[7] = Byte7;

  /* Start the Transmission process */
  if ( HAL_FDCAN_AddMessageToTxFifoQ(&hfdcan1, &FDCAN_TxHeader, FDCAN_TxData) != HAL_OK )
  {
    /* Transmission request Error */
    // Error_Handler();

    HAL_Delay (2); // request to fast add a small delay
  }
}

/**
 * @brief  Send a CANFD-message on CAN-bus, DLC(=12)
 *         Is the CANID smaller than 0x800 the standard ID frame format is used, otherwise the extended ID frame format is used.
 * @param  CANID: specifies the CANID with which the message is to be sent
 *         Byte0: Data in Byte 0
 *         Byte1: Data in Byte 1
 *         Byte2: Data in Byte 2
 *         Byte3: Data in Byte 3
 *         Byte4: Data in Byte 4
 *         Byte5: Data in Byte 5
 *         Byte6: Data in Byte 6
 *         Byte7: Data in Byte 7
 *         Byte8: Data in Byte 8
 *         Byte9: Data in Byte 9
 *         Byte10: Data in Byte 10
 *         Byte11: Data in Byte 11
 *         Byte12: Data in Byte 12
 * @retval none
 */
void SendCAN_FDMessage_DLC12 ( uint32_t CANID, uint8_t Byte0, uint8_t Byte1, uint8_t Byte2, uint8_t Byte3, uint8_t Byte4, uint8_t Byte5, uint8_t Byte6,
    uint8_t Byte7, uint8_t Byte8, uint8_t Byte9, uint8_t Byte10, uint8_t Byte11 )
{
  /* Prepare Tx Header */
  FDCAN_TxHeader.FDFormat = FDCAN_FD_CAN;
  FDCAN_TxHeader.TxFrameType = FDCAN_DATA_FRAME;
  FDCAN_TxHeader.ErrorStateIndicator = FDCAN_ESI_ACTIVE;
  FDCAN_TxHeader.BitRateSwitch = FDCAN_BRS_OFF;
  FDCAN_TxHeader.TxEventFifoControl = FDCAN_NO_TX_EVENTS;
  FDCAN_TxHeader.MessageMarker = 0;
  FDCAN_TxHeader.Identifier = CANID;
  FDCAN_TxHeader.IdType = FDCAN_STANDARD_ID;
  if ( CANID >= 0x800 )
    FDCAN_TxHeader.IdType = FDCAN_EXTENDED_ID;
  FDCAN_TxHeader.DataLength = FDCAN_DLC_BYTES_12;

  /* Fill Data */
  FDCAN_TxData[0] = Byte0;
  FDCAN_TxData[1] = Byte1;
  FDCAN_TxData[2] = Byte2;
  FDCAN_TxData[3] = Byte3;
  FDCAN_TxData[4] = Byte4;
  FDCAN_TxData[5] = Byte5;
  FDCAN_TxData[6] = Byte6;
  FDCAN_TxData[7] = Byte7;
  FDCAN_TxData[8] = Byte8;
  FDCAN_TxData[9] = Byte9;
  FDCAN_TxData[10] = Byte10;
  FDCAN_TxData[11] = Byte11;

  /* Start the Transmission process */
  if ( HAL_FDCAN_AddMessageToTxFifoQ(&hfdcan1, &FDCAN_TxHeader, FDCAN_TxData) != HAL_OK )
  {
    /* Transmission request Error */
    // Error_Handler();

    HAL_Delay (2); // request to fast add a small delay
  }
}

/**
 * @brief  Read Voltage from Analog input
 * @param  Channel: [IN1_ANALOG, IN2_ANALOG, IN3_ANALOG]
 *         MeasureingRange: [INPUT_RANGE_10_VOLTS, INPUT_RANGE_30_VOLTS]
 * @retval Voltage: [mV], 0xFFFF@ wrong channel, 0xFFFE@ timeout conversation ADC
 */
uint16_t ReadInputAnalog_mV ( uint32_t Channel, uint8_t MeasureingRange )
{
  uint16_t u16DR = 0;
  ADC_ChannelConfTypeDef sConfig = {0};

  if ( (Channel != IN1_ANALOG) && (Channel != IN2_ANALOG) && (Channel != IN3_ANALOG) )
    return 0xFFFF; // select wrong channel

  if ( MeasureingRange == INPUT_RANGE_10_VOLTS )
    HAL_GPIO_WritePin(Messbereichsumschaltung_GPIO_Port, Messbereichsumschaltung_Pin, GPIO_PIN_RESET);
  else
    HAL_GPIO_WritePin(Messbereichsumschaltung_GPIO_Port, Messbereichsumschaltung_Pin, GPIO_PIN_SET);
  HAL_Delay(2); // delay for switch FET

  /** Configure Regular Channel
   */
  sConfig.Channel = Channel;
  sConfig.Rank = ADC_REGULAR_RANK_1;
  sConfig.SamplingTime = ADC_SAMPLETIME_92CYCLES_5;
  sConfig.SingleDiff = ADC_SINGLE_ENDED;
  sConfig.OffsetNumber = ADC_OFFSET_NONE;
  sConfig.Offset = 0;
  if ( HAL_ADC_ConfigChannel(&hadc1, &sConfig) != HAL_OK )
  {
    Error_Handler();
  }

  HAL_ADC_Start(&hadc1);

  if ( HAL_ADC_PollForConversion(&hadc1, 100) == HAL_OK )
  {
    u16DR = HAL_ADC_GetValue(&hadc1);
    HAL_ADC_Stop(&hadc1);

    if ( MeasureingRange == INPUT_RANGE_10_VOLTS )
      return (uint16_t) ((uint32_t) 10284 * (uint32_t) u16DR / 4096);
    else
      return (uint16_t) ((uint32_t) 30304 * (uint32_t) u16DR / 4096);
  }
  else
    return 0xFFFE; // Error
}


/**
 * @brief  Read pin state IN4 or IN5
 * @param  Channel: [IN4_DIGITAL, IN5_DIGITAL]
 * @retval Pin State: 1@High, 0@Low, 0xFF@Wrong channel
 */
uint8_t ReadInputDigital ( uint8_t Channel )
{
  switch ( Channel )
  {
    default:
      return 0xFF; // select wrong channel
      // break;

    case IN4_DIGITAL:
      if ( HAL_GPIO_ReadPin(DIN4_T2_1_GPIO_Port, DIN4_T2_1_Pin) == GPIO_PIN_SET )
        return 1;
      else
        return 0;
      // break;

    case IN5_DIGITAL:
      if ( HAL_GPIO_ReadPin(DIN5_T2_2_GPIO_Port, DIN5_T2_2_Pin) == GPIO_PIN_SET )
        return 1;
      else
        return 0;
      // break;
  }
}

/**
 * @brief  Set output pin high or low
 * @param  Channel: [OUT1_HS, OUT2_HS, OUT3_HS, OUT4_HS]
 *         State: OUT_OFF@0, OUT_ON@1
 * @retval none
 */
void WriteDigitalOutput ( uint8_t Channel, GPIO_PinState State )
{
  switch ( Channel )
  {
    default:
      // select wrong channel
      break;

    case OUT1_HS:
      HAL_GPIO_WritePin(OUT1_GPIO_Port, OUT1_Pin, State);
      break;

    case OUT2_HS:
      HAL_GPIO_WritePin(OUT2_GPIO_Port, OUT2_Pin, State);
      break;

    case OUT3_HS:
      HAL_GPIO_WritePin(OUT3_GPIO_Port, OUT3_Pin, State);
      break;

    case OUT4_HS:
      HAL_GPIO_WritePin(OUT4_GPIO_Port, OUT4_Pin, State);
      break;
  }
}

/**
 * @brief  Set duty cycle for OUT5
 * @param  Channel: OUT5_LS
 *         DutyCycle_Percent: [0..100]
 * @retval none
 */
void WriteDigitalOutputPWM_Percent ( uint8_t Channel, uint16_t DutyCycle_Percent )
{
  if (Channel != OUT5_LS)
    return; // select wrong channel
  if (DutyCycle_Percent > 100)
    return; // percent over 100%

  // set new value
  htim5.Instance->CCR3 = (DutyCycle_Percent * htim5.Instance->ARR) / 100;
}

/**
 * @brief  Set duty cycle for LED
 * @param  DutyCycle_Percent: [0..100]
 * @retval none
 */
void WriteLED_Percent ( uint16_t DutyCycle_Percent )
{
  if (DutyCycle_Percent > 100)
    return; // percent over 100%

  htim8.Instance->CCR4 = (DutyCycle_Percent * htim8.Instance->ARR )/ 100;
}

/**
  * @brief  Read from EEPROM
  * @param  BaseAddress: EEPROM start address
  * @retval Byte
  */
uint8_t EEPROM_Read_Byte ( uint16_t BaseAddress )
{
  uint8_t DataFromEEPROM;
  HAL_I2C_Mem_Read(&hi2c2, EEPROM_ADDRESS, BaseAddress, I2C_MEMADD_SIZE_16BIT, &DataFromEEPROM , 1, EEPROM_TIMEOUT);
  return DataFromEEPROM;
}

/**
  * @brief  Write to EEPROM
  * @param  BaseAddress: EEPROM start address
  * @param  Data: data source for write
  * @retval none
  */
void EEPROM_Write_Byte ( uint16_t BaseAddress, uint8_t Data )
{
  HAL_I2C_Mem_Write(&hi2c2, EEPROM_ADDRESS, BaseAddress, I2C_MEMADD_SIZE_16BIT, &Data, 1, EEPROM_TIMEOUT);
  HAL_Delay(EEPROM_WRITE_TIME);
}

